/*
	To build use the following gcc statement 
	(assuming you have the d2xx library in the /usr/local/lib directory).
	gcc -o adc main.c -L. -lftd2xx -Wl,-rpath,/usr/local/lib
*/

#include <stdio.h>
#include <unistd.h>
#include "../ftd2xx.h"

#define RETRY_COUNT 10	// Read retry counter

#define SCK 0x01
#define SDO 0x02
#define SDI 0x04
#define SS  0x08

#define CLK_100K 299
#define CLK_200K 149
#define CLK_400K 74
#define CLK_1M 29

#define BUFFER_SIZE 64


FT_HANDLE	ftHandle;
char iobuf[BUFFER_SIZE];
unsigned int iobufindex;

// Function declarations
void Add2Buffer(char data);
char send_buffer(FT_HANDLE devhandle);
char read_buffer(FT_HANDLE devhandle, DWORD bytes2read);
char sync2mpsse(FT_HANDLE devhandle);
unsigned short read_adc(FT_HANDLE devhandle);


//--------------------------------------------------------------------
int main(int argc, char *argv[])
{
	FT_STATUS	ftStatus;
	int iport;
	
	if(argc > 1) {
		sscanf(argv[1], "%d", &iport);
	}
	else {
		iport = 0;
	}
	
	ftStatus = FT_Open(iport, &ftHandle);
	if(ftStatus != FT_OK)
	{
		/* 
			This can fail if the ftdi_sio driver is loaded
		 	use lsmod to check this and rmmod ftdi_sio to remove
			also rmmod usbserial
		 */
		printf("FT_Open(%d) failed, with error %d.\n", iport, (int)ftStatus);
		printf("Use lsmod to check if ftdi_sio (and usbserial) are present.\n");
		printf("If so, unload them using rmmod, as they conflict with ftd2xx.\n");
		return 1;
	}
	printf("Port: (%d) open - OK\n", iport);

	ftStatus = FT_SetBaudRate(ftHandle, 12000000);
	if(ftStatus != FT_OK)
	{
		printf("Failed to set baud rate (error %d)\n", (int)ftStatus);
		goto error;
	}
	printf("FT_SetBaudRate to 12Mbps - OK\n");

	ftStatus = FT_SetFlowControl(ftHandle, FT_FLOW_RTS_CTS, 0x11, 0x13);
	if(ftStatus != FT_OK)
	{
		printf("Failed to set flow control (error %d)\n", (int)ftStatus);
		goto error;
	}
	printf("FT_SetFlowControl to FT_FLOW_RTS_CTS - OK\n");
	
	ftStatus = FT_SetBitMode(ftHandle, 0x00, FT_BITMODE_MPSSE);
	if(ftStatus != FT_OK)
	{
		printf("Failed to set bit mode (error %d)\n", (int)ftStatus);
		goto error;
	}
	printf("FT_SetBitMode to MPSSE - OK\n");
	
	if(sync2mpsse(ftHandle))
	{
		printf("sync2mpsse failed!\n");
		goto error;
	}
	printf("sync2mpsse - OK\n");

	iobufindex = 0;
	Add2Buffer(0x80);	// Set GPIOL
	Add2Buffer(0xFF);	// All high
	Add2Buffer(SCK | SDO | SS);	// as outputs
	Add2Buffer(0x82);	// Set GPIOH
	Add2Buffer(0xFF);	// All high
	Add2Buffer(0x00);	// as inputs
	Add2Buffer(0x82);	// Disable clock divide by 5. Master clock = 60MHz
	Add2Buffer(0x86);	// Set clock frequency
	Add2Buffer(CLK_1M & 0xFF);	// ValueL
	Add2Buffer((CLK_1M >> 8) & 0xFF);	// ValueH
	Add2Buffer(0x85);	// Turn loopback off
	if(send_buffer(ftHandle))
	{
		printf("Write failed!\n");
		goto error;
	}
	printf("MPSSE initialized - OK\n");
	
	ftStatus = FT_Purge(ftHandle,(FT_PURGE_RX | FT_PURGE_TX));
	if(ftStatus != FT_OK)
	{
		printf("Failed to purge Rx and Tx buffers (error %d)\n", (int)ftStatus);
		goto error;
	}
	printf("Rx/Tx buffer purge - OK\n\n");

	do
	{
		printf("ADC read = %d\n", read_adc(ftHandle));
		printf("Press 'q' to quit or 'Enter' to read again\n");
	}
	while(getchar() != 'q');

	FT_Close(ftHandle);
	return 0;
error:
	FT_Close(ftHandle);
	return 1;
}
//--------------------------------------------------------------------
void Add2Buffer(char data)
{
	iobuf[iobufindex] = data;
	iobufindex++;
}
//--------------------------------------------------------------------
char send_buffer(FT_HANDLE devhandle)
{
	DWORD written;
	
	if(FT_Write(devhandle, &iobuf, iobufindex, &written) == FT_OK)
	{
		iobufindex = (iobufindex - written);
		if(iobufindex == 0)
		{
			return 0;
		}
	}
	return 1;
}
//--------------------------------------------------------------------
char read_buffer(FT_HANDLE devhandle, DWORD bytes2read)
{
	short retry_count = RETRY_COUNT;
	DWORD data_queue;

	while(retry_count)	// Wait for data to become available
	{
		if(FT_GetQueueStatus(devhandle, &data_queue) != FT_OK)
		{
			return 1;
		}
		if(data_queue >= bytes2read)
		{
			break;
		}
		retry_count--;
	}
	//-----------------------------
	if(FT_Read(devhandle, &iobuf, data_queue, &iobufindex) == FT_OK)
	{
		if(iobufindex >= bytes2read)
		{
			return 0;
		}
	}
	return 1;
}
//--------------------------------------------------------------------
char sync2mpsse(FT_HANDLE devhandle)
{
	DWORD i;

	iobufindex = 0;
	Add2Buffer(0xAA);
	Add2Buffer(0x87);	// Send Immediate
	if(send_buffer(devhandle))
	{
		return 1;
	}
	sleep(1);
	if(read_buffer(devhandle, 2))
	{
		return 1;
	}
	for(i = 0; i < iobufindex; i++)
	{	
		if((iobuf[i] == 0xFA) && (iobuf[i+1] == 0xAA))
		{
			return 0;
		}
	}
	return 1;
}
//--------------------------------------------------------------------
unsigned short read_adc(FT_HANDLE devhandle)
{
	unsigned short result;

	iobufindex = 0;

	Add2Buffer(0x80);		// Set GPIOL
	Add2Buffer(0xFF & (~SS));	// All high, SS-Low
	Add2Buffer(SCK | SDO | SS);	// as outputs
	Add2Buffer(0x20);		// Do Read
	Add2Buffer(0x01);		// 2 bytes
	Add2Buffer(0x00);		// 
	Add2Buffer(0x80);		// Set GPIOL
	Add2Buffer(0xFF);		// All high
	Add2Buffer(SCK | SDO | SS);	// as outputs
	Add2Buffer(0x87);		// Send Immediate
	if(send_buffer(devhandle))
	{
		return 0xFFFF;
	}
	if(read_buffer(devhandle, 2))
	{
		return 0xFFFF;
	}
	if(iobufindex < 2)
	{
		return 0xFFFF;
	}
	result = (iobuf[0] & 0xFF);
	result = ((result << 8) | (iobuf[1] & 0xFF));
	result = (result >> 4);
	return (result);
}

